@@ -3,6 +3,7 @@ from collections import defaultdict, OrderedDict, Counter
 from IPython.display import HTML, display
 from graphviz import Graph
 from copy import deepcopy
+from maquinas.exceptions import NoSymbolInMachine
 
 class PartialProduction():
     def __init__(self,head,body,lps=None):
@@ -400,14 +401,14 @@ def _update_stack(stack,root,partials,nchars,terminals,prev=None,depth=0,anceste
             stack.insert(0,(root,current,[],partials,nchars+1,depth+1,ancesters))
         elif current[0]['label'][0]=='ε':
             stack.insert(0,(root,current,[],partials,nchars,depth+1,ancesters))
-
+        else:
+            raise NoSymbolInMachine(current[0]['label'][0],terminals)
     if len(branches)>0:
         root_,current_=_deepcopy(root,current[0])
         if current_ is None:
             partials.append((current,branches))
             current[0]['current']=False
         else:
-            partials.insert(0,[])
             stack.append((root,current,branches[0],partials,nchars,depth+1,ancesters))
             for ix in range(1,len(branches)):
                 root_,current_=_deepcopy(root,current[0])
@@ -435,6 +436,7 @@ def _graph_tree(f,tree,parent,
         full=False,
         show_id=False,
         **params):
+    tree=list(tree)
     if len(tree)>1:
         u,children=tree
     else:
