@@ -16,8 +16,19 @@ class TuringMachine():
     """Turing machine"""
 
     def __init__(self, Q=[], sigma=[], gamma=[], B='𝖁',q_0=None,A=[], delta={}, force=False):
+        """Common class for Turing Machine
+
+        :param Q: Ordered set of states (default is empty).
+        :param sigma: Ordered set of terminal symbols (default is empty).
+        :param gamma: Ordered set of tape symbols (default is empty).
+        :param B: Blank symbol (default 𝖁).
+        :param q_0: Initial state (default None).
+        :param A: Set of acceptor states (default is empty).
+        :param delta: List of transitions with the form tupla of tupla of q_i and a, and the list of q_f, a symbol and direction (default is empty).
+        :param force: If True and states or symbols do not exists create them (default is False).
+        :type force: bool
+        """
         self.sigma=OrderedSet()
-        self.sigma.add('ε')
         self.B=B
         self.i=0
         self.gamma=OrderedSet()
@@ -69,7 +80,10 @@ class TuringMachine():
             return 0
 
     def _ntsymbol(self,t):
-        return self.gamma.index(self._filter(t))
+        try:
+            return self.gamma.index(self._filter(t))
+        except KeyError as c:
+            raise DoesNotExistsSymbol(c)
 
     def _state(self,nq):
         return self.Q.items[nq]
@@ -81,18 +95,34 @@ class TuringMachine():
         return self.gamma.items[nz]
 
     def _status(self,status,states={},symbols={}):
-        return "|".join(f"{states.get(self._state(s),self._state(s))}" for s,_,_,_ in status)
+        return "|".join(f"{states.get(s,s)}" for s,_,_,_ in status)
 
     def states(self):
+        """ Gets states
+
+        :returns: States of machine
+        :rtype: list"""
         return list(self.Q)
 
     def symbols(self):
+        """ Gets terminal symbols
+
+        :returns: Terminal symbols of machine
+        :rtype: list"""
         return list(self.sigma)
 
     def tsymbols(self):
+        """ Gets tape symbols
+
+        :returns: Tape symbols of machine
+        :rtype: list"""
         return list(self.gamma)
 
     def tokens(self,r):
+        """ Gets  tokens for tape
+
+        :returns: Tape symbols for machine
+        :rtype: list"""
         return re_tape.findall(r)
 
     def _transition(self,nq_i,nt,nq_f,nt_,Dir):
@@ -110,6 +140,12 @@ class TuringMachine():
             return set()
 
     def delta(self,states):
+        """ Applies delta function
+
+        :param states: Internal state composed by (state, tape negative positions, tape positive positions, and (current position, symbol)
+        :param a: Symbol
+
+        :returns: Destination state"""
         states_=set()
         for nq,tn,tp,(c,a) in states:
             if c<0:
@@ -141,23 +177,49 @@ class TuringMachine():
         return states_
 
     def _tape(self,nt,pt):
-        return [t for t in reversed(nt)]+[t for t in pt]
+        return [t for t in reversed(tuple(nt))]+[t for t in pt]
 
     def delta_stepwise(self,w,q=None,max_steps=0):
+        """ Applies a step of delta extended function
+
+        :param w: String
+        :param q: Internal state where to start (default is initial state)
+        :param max_steps: Maximun number of steps to consider
+
+        :returns: Tuple with state of precessing at step, consisting of: state, left tape, right tape, (position, processed string)"""
         if q is None:
             states=set([(self._nstate(self.q_0),(),tuple(self._ntsymbol(a) for a in w),(0,""))])
-            yield states
+            yield self._index2label(states)
         steps=0
+        A=set(self._nstate(a) for a in self.A)
         while len(states)>0 and len(set([q for q,_,_,_ in states]).intersection(self.A))==0:
             states=self.delta(states)
             steps+=1
-            yield states
+            yield self._index2label(states)
+            if set([s for s,_,_,_ in states]).intersection(A):
+                break
             if max_steps and steps>=max_steps:
                 break
 
+    def create_initial_istate(self,w):
+        """ Creates an initial internal state 
+
+        :param w: string for the tape
+
+        :return: Returns internal initial state"""
+        return set([(self._nstate(self.q_0),(),tuple(self._ntsymbol(a) for a in w),(0,""))])
+
     def delta_extended(self,states,w,max_steps=None):
+        """ Applies delta extended function
+
+        :param states: Internal states
+        :param w: String
+        :param max_steps: Maximum number of states
+
+        :returns: Returns internal state after processing the full string"""
+
         if states is None:
-            states=set([(self._nstate(self.q_0),(),tuple(self._ntsymbol(a) for a in w),(0,""))])
+            states=self.create_initial_istate(w)
         steps=0
         while len(states)>0 and len(set([self._state(q) for q,_,_,_ in states]).intersection(self.A))==0:
             states=self.delta(states)
@@ -165,42 +227,96 @@ class TuringMachine():
             res=[(q,tn,tp,c) for q,tn,tp,c in states]
             if max_steps and steps>=max_steps:
                 return []
-        return res
+        return self._index2label(res)
+
+    def _index2label(self,states_):
+        return [(self._state(q),
+            tuple(self._tsymbol(s) for s in l),
+            tuple(self._tsymbol(s) for s in r),
+            c
+            )
+            for q,l,r,c in states_]
+
+    def _label2index(self,states):
+        return [(self._nstate(q),
+            tuple(self._ntsymbol(s) for s in l),
+            tuple(self._ntsymbol(s) for s in r),
+            c
+            )
+            for q,l,r,c in states]
+
 
     def items(self):
+        """ Iterator over the transitions
+
+        :returns: Yeilds a tuple transition"""
         for nq_i,val in self.ttable.items():
             for nt,nq_fs in val.items():
                 for (nq_f,nt_,Dir) in nq_fs:
                     yield self._transition(nq_i,nt,nq_f,nt_,Dir)
 
     def step(self,states):
-        return self.delta(states)
+        return self._index2label(self.delta(states))
 
     def get_transition(self,q,a):
-        return self._state(self._get_transition(self._nstate(q),self._nsymbol(a)))
+        """ Gets the destintion state or states for state, terminal symbol and stack symbol
 
-    def add_transition(self,q_i,t,qs):
-        nq_i=self.add_state(q_i)
-        nt=self.add_tsymbol(t)
+        :param q: Source state
+        :param a: Terminal symbol
+        :returns: Destination state or states"""
+
+        qs=self._get_transition(self._nstate(q),self._nsymbol(a))
+        return [ (self._state(s),self.tsymbol(a),d) for s,a,d in qs]
+
+    def add_transition(self,q_i,t,qs,force=False):
+        """ Adds a transition
+
+        :param q_i: Source state
+        :param t: Tape symbol
+        :param q_s: Destination state (q_f,t_2,dir)
+        :param force: Force creation of elements
+        :returns: None"""
+        try:
+            nq_i=self.add_state(q_i)
+        except AlreadyExistsState:
+            nq_i=self._nstate(q_i)
+        try:
+            nt=self.add_tsymbol(t)
+        except AlreadyExistsSymbol:
+            nt=self._ntsymbol(t)
         qs=[(self._nstate(q),self._ntsymbol(t),Dir) for q,t,Dir in qs]
+
+        if nq_i in self.ttable and \
+            nt in self.ttable[nq_i]:
+                raise AlreadyExistsTMTransition(q_i,t,self)
+
         if not nq_i in self.ttable:
             self.ttable[nq_i]={}
         if not nt in self.ttable[nq_i]:
             self.ttable[nq_i][nt]=set()
         self.ttable[nq_i][nt].update(qs)
 
-    def valitate(self):
-        pass
-
     def add_state(self,q,initial=False):
+        """ Adds a state
+
+        :param q: State or states
+        :param initial: Set state as a initial
+        :returns: Indixes of state or states"""
         if initial:
             self.q_0=q
+        if q in self.Q:
+            raise AlreadyExistsState(q)
         if isinstance(q,(set,list)):
             return set(self.Q.add(q_) for q_ in q)
         else:
             return self.Q.add(q)
 
     def add_next_state(self,initial=False):
+        """ Adds a state with a infered name based on the number of states q_max. If the name state is already defined it looks the following integer available.
+
+        :param q: State or states
+        :param initial: Set state as a initial
+        :returns: Next state generated and integer"""
         max_ix=len(self.Q)
         while f"q_{max_ix}" in self.Q:
             max_ix+=1
@@ -211,227 +327,127 @@ class TuringMachine():
         return q,max_ix
 
     def add_symbol(self,a):
+        """ Adds a symbol
+
+        :param a: Symbol
+        :returns: Indixes of symbol"""
+        if a in self.gamma:
+            raise AlreadyExistsSymbol(a)
         return self.sigma.add(a)
         return self.gamma.add(a)
 
     def add_tsymbol(self,t):
+        """ Adds a tape symbol
+
+        :param t: Tape xymbol
+        :returns: Indixes of symbol"""
+        if t in self.gamma:
+            raise AlreadyExistsSymbol(t)
         return self.gamma.add(t)
 
     def set_initial_state(self,q,force=False):
+        """ Sets an initial state
+
+        :param q: State
+        :param force: If not defined it creates it (default is False)
+        :returns: None"""
         if q is None:
             self.q_0=None
             return None
-        if force and not q in self.Q:
-            self.add_state(q)
+        if not q in self.Q:
+            if force:
+                self.add_state(q)
+            else:
+                raise DoesNotExistsState(q)
         self.q_0=q
 
-    def set_initial_qsymbol(self,z,force=False):
-        if z is None:
-            self.Z_0='Z₀'
-        else:
-            if force and not z in self.gamma:
-                self.gamma.add(z)
-            self.Z_0=z
-        return self.Z_0
-
     def get_initial_state(self):
+        """ Gets an initial state
+
+        :returns: State"""
         return self.q_0
 
-    def autorename(self,start=0,avoid=[],copy=True):
-        replacements=[(q,f'q_{start+ix}') for q,ix in self.Q.map.items()]
-        ix=len(self.Q)+start
-        new_A=set()
-        while len(replacements)>0:
-            old,new=replacements.pop(0)
-            if new == old:
-                if old in self.A:
-                    new_A.add(new)
-                if old == self.q_0:
-                    self.q_0=new
-                continue
-            elif new in self.Q or new in avoid:
-                replacements.append((old,f'q_{ix}'))
-                ix+=1
-            else:
-                self.replace_state(old,new)
-                if old in self.A:
-                    new_A.add(new)
-                if old == self.q_0:
-                    self.q_0=new
-        self.A=new_A
-
-    def remove_states(self,states):
-        Q_=OrderedSet()
-        ttable_={}
-        for q in self.Q:
-            if not q in states:
-                Q_.add(q)
-
-        ttable_={}
-        for nq_i,tt in self.ttable.items():
-            q_i=self.Q.items[nq_i]
-            if q_i in states:
-                continue
-            nq_i_=Q_.map[q_i]
-            for na,nqs in tt.items():
-                if isinstance(nqs,set):
-                    qs=[self.Q.items[q] for q in nqs ]
-                    res=set([Q_.index(q) for q in qs if not q in states])
-                else:
-                    q=self.Q.items[nqs]
-                    if q in states:
-                        continue
-                    res=Q_.index(q)
-                try:
-                    ttable_[nq_i_][na]=res
-                except KeyError:
-                    ttable_[nq_i_]={}
-                    ttable_[nq_i_][na]=res
-        self.Q=Q_
-        self.ttable=ttable_
-
-    def remove_sink_states(self):
-        Q_=OrderedSet()
-        idx_remove=set()
-        for nq,q in enumerate(self.Q):
-            if not q in self.A:
-                if not nq in self.ttable:
-                    idx_remove.add(nq)
-                    continue
-            destination=set()
-            # If state is loopy
-            for na,a in enumerate(self.sigma):
-                try:
-                    res=self.ttable[nq][na]
-                except KeyError:
-                    continue
-                if isinstance(res,set):
-                    destination=destination | res
-                else:
-                    destination.add(res)
-            if len(destination)==1 and nq in destination and q not in self.A:
-                idx_remove.add(nq)
-                continue
-            Q_.add(q)
-
-        ttable_={}
-        for nq_i_,q_i in enumerate(Q_):
-            nq_i=self.Q.map[q_i]
-            for na,a in enumerate(self.sigma):
-                try:
-                    res=self.ttable[nq_i][na]
-                except KeyError:
-                    continue
-                if isinstance(res,set):
-                    qs=[self.Q.items[nq] for nq in res if not nq in idx_remove]
-                    res=set([Q_.index(q) for q in qs])
-                else:
-                    if res in idx_remove:
-                        continue
-                    q=self.Q.items[res]
-                    res=Q_.index(q)
-                try:
-                    ttable_[nq_i_][na]=res
-                except KeyError:
-                    ttable_[nq_i_]={}
-                    ttable_[nq_i_][na]=res
-        self.Q=Q_
-        self.ttable=ttable_
-
-    def get_initial_state(self):
-        return self._get_state_label(self._get_initial_state())
-
     def set_aceptors(self,A,force=False):
+        """ Sets aceptors states
+
+        :param A: States
+        :param force: If not defined it creates it (default is False)
+        :returns: None"""
         if force:
             self.add_state(A)
         self.A=set(A)
 
     def stepStatus(self,status):
+        """ Gives a step and calculates new status for Simulation
+
+        :param Status: Status
+        :returns: None"""
         if status.state is None:
-            states=set([(self._nstate(self.q_0),(),(),0,"")])
+            states=self._index2label(self.create_initial_istate(status.string))
         else:
             states=status.state
-
-        states=self.step(states)
+        states=self.step(self._label2index(states))
         status.position+=1
         status.step+=1
         status.state=states
 
-    def accept(self,states):
-        final=set([self._state(q) for q,_,_,_ in states])
+    def accepts(self,w,max_steps=0):
+        """ Checks if string is accepted
+
+        :param w: String
+        :returns: None"""
+        try:
+            return self.acceptor(self.delta_extended(None,w,max_steps=max_steps))
+        except:
+            return False
+
+    def acceptor(self,states):
+        """ Checks if state is an acceptor state
+
+        :param states: State or states
+        :type: Set
+
+        :returns: None"""
+        final=set([q for q,_,_,_ in states])
         if bool(final.intersection(self.A)):
             return True
         return False
 
-    def replace_symbol(self,old,new):
-        ix=self.sigma.index(old)
-        del self.sigma.map[old]
-        self.sigma.map[new]=ix
-        self.sigma.items[ix]=new
+    def save_img(self,filename,q_c=set(),a_c=set(),q_prev=set(),symbols={},states={},format='svg',dpi="60.0",string=None,status=None):
+        """ Saves machine as an image
 
-    def replace_state(self,old,new):
-        if new in self.Q:
-            raise AlreadyExistsState(new)
-        ix=self.Q.index(old)
-        del self.Q.map[old]
-        self.Q.map[new]=ix
-        self.Q.items[ix]=new
-        if old == self.q_0:
-            self.q_0=new
+            :param filename: Filename of image
+            :param q_c: Set of current states to be highlited (default is empty)
+            :param a_c: Set of current symbols to be highlited (default is empty)
+            :param q_prev: Set of previos states to be highlited (default is empty)
+            :param symbols: Replacements of the symbols to show (default is empty)
+            :param states: Replacements of the states to show (default is empty)
+            :param format: Format of image (default is svg)
+            :param dpi: Resolution of image (default is "90.0")
+            :param string: Label of string being analysed (default is None)
+            :param stack: Status of the stack (default is None)
+            :param status: Status of the TM (default is None)
 
-    def add_error_state(self,e_label="q_E"):
-        self.add_state(e_label)
-        ne=self.Q.index(e_label)
-        for nq,q in enumerate(self.Q):
-            for na,a in enumerate(self.sigma):
-                if not nq in self.ttable:
-                    self.ttable[nq]={}
-                    self.ttable[nq][na]=ne
-                elif not na in self.ttable[nq]:
-                    self.ttable[nq][na]=ne
-
-    def reachable_states(self):
-        # https://en.wikipedia.org/wiki/DFA_minimization#Unreachable_states
-        reachable=set([self.q_0])
-        new=set([self.q_0])
-        while len(new)>0:
-            temp=set()
-            for q in new:
-                for a in self.sigma:
-                    try:
-                        states=self.get_transition(q,a)
-                        if isinstance(states,set):
-                            temp.update(states)
-                        else:
-                            temp.add(states)
-                    except DoesNotExistsTransition:
-                        pass
-            new=temp.difference(reachable)
-            reachable=reachable.union(new)
-        return reachable
-
-    def unreachable_states(self):
-        return set(self.Q.difference(self.reachable_states()))
-
-    def remove_unreachable(self):
-        self.remove_states(self.unreachable_states())
-
-
-    def save_img(self,filename,q_c=set(),a_c=set(),q_prev=set(),symbols={},states={},format='svg',dpi="90.0",string=None,status=None):
+            :returns: None"""
         dot=self.graph(q_c=q_c,a_c=a_c,q_prev=q_prev,symbols=symbols,states=states,format=format,dpi=dpi,string=string,status=status)
         dot.render(filename,format="png",cleanup=True)
 
     def states2string(self,states):
+        """ Renders srting with the state of the TM
+
+        :returns: String tiwh the states of the TM"""
         res=[]
         for q,nt,pt,(c,a) in states:
             tape=self._tape(nt,pt)
-            res.append("{}, … {} _{}_ {} …".format(self._state(q),\
-                "".join([ self._tsymbol(t) for t in tape[:c]]),\
-                self._tsymbol(tape[c]),\
-                "".join([ self._tsymbol(t) for t in tape[c+1:]])))
+            res.append("{}, … {} _{}_ {} …".format(q,\
+                "".join([ t for t in tape[:c]]),\
+                tape[c],\
+                "".join([ t for t in tape[c+1:]])))
         return "|".join(res)
 
     def summary(self):
+        """ Producrs summary of the PDA
+        :returns: List with summary"""
         info= [
          "States  : "+", ".join(self.states()),
          "Sigma   : "+", ".join(self.symbols()),
@@ -441,17 +457,38 @@ class TuringMachine():
          "Transitions:\n"+"\n".join(f" {q_i},{t}/{t_} → {q_f},{Dir}" for (q_i,t),(q_f,t_,Dir) in self.items())]
         return "\n".join(info) 
 
-    def graph(self,q_c=set(),a_c=set(),q_prev=set(),states={},symbols={},format="svg",dpi="60.0",string=None,stack=[],status=None,one_arc=True,finished=False):
+    def print_summary(self):
+        """ Prints a summary of the PDA
+        """
+        print(self.summary())
+
+
+    def graph(self,q_c=set(),a_c=set(),q_prev=set(),symbols={},states={},format="svg",dpi="60.0",string=None,status=None,one_arc=True,finished=False):
+        """ Graphs TM
+
+        :param q_c: Set of current states to be highlited (default is empty)
+        :param a_c: Set of current symbols to be highlited (default is empty)
+        :param q_prev: Set of previos states to be highlited (default is empty)
+        :param symbols: Replacements of the symbols to show (default is empty)
+        :param states: Replacements of the states to show (default is empty)
+        :param format: Format of image (default is svg)
+    :param dpi: Resolution of image (default is "60.0")
+        :param string: Label of string being analysed (default is None)
+        :param status: Status of the TM (default is None)
+        :param one_arc: Graph one arc in case of multiple transitions (default is True)
+        :param finished: If has pass through final state (default is False)
+
+        :returns: Returns Digraph object from graphviz"""
         if len(q_c)==0:
             states_=[(self._nstate(self.q_0),(),(),(0,""))]
         else:
             states_=q_c
 
-        f=Digraph(comment="PDAs",format=format)
+        f=Digraph(comment="TM",format=format)
         f.attr(rankdir='LR',dpi=dpi)
         for i,(q_c_,nt,pt,(c,a)) in enumerate(states_):
             if len(q_c)>0:
-                q_c_=set([self._state(q_c_)])
+                q_c_=set([q_c_])
             else:
                 q_c_=[]
             with f.subgraph(name=f'cluster_{i}') as f_:
@@ -487,9 +524,9 @@ class TuringMachine():
             cells=[]
             for i,c in enumerate(tape):
                 if i==pos:
-                    cells.append(f'<TD BGCOLOR="{color_state}">{symbols.get(self._tsymbol(c),self._tsymbol(c))}</TD>')
+                    cells.append(f'<TD BGCOLOR="{color_state}">{symbols.get(c,c)}</TD>')
                 else:
-                    cells.append(f'<TD>{symbols.get(self._tsymbol(c),self._tsymbol(c))}</TD>')
+                    cells.append(f'<TD>{symbols.get(c,c)}</TD>')
             label_tape=f"< <TABLE BORDER='0' CELLBORDER='1' SIDES='TBRL'><TR>{' '.join(cells)}</TR></TABLE> >"
 
         if label_tape:
@@ -538,6 +575,15 @@ class TuringMachine():
 
 
     def table(self,symbols={},states={},q_order=None,s_order=None,color_final="#32a852",empty_symbol="∅"):
+        """ Creates an HTML object for the table of the PDA
+
+        :param symbols: Replacements of the symbols to show (default is empty)
+        :param states: Replacements of the states to show (default is empty)
+        :param  q_order: Order to use for states
+        :param  s_order: Order to use for symbols
+        :param color_final: RGB string for color of final state (default is "#32a852")
+
+        :returns: Display object for IPython"""
         if not s_order:
             s_order=list(self.gamma)
             s_order.sort()
@@ -564,7 +610,19 @@ class TuringMachine():
         table+="</table>"
         return display(HTML(table))
 
-    def save_gif(self,w,filename="tm.gif",symbols={},states={},dpi="300",show=True,loop=0,max_steps=1000):
+    def save_gif(self,w,filename="tm.gif",symbols={},states={},dpi="90",show=True,loop=0,duration=500,max_steps=1000):
+        """ Saves an animation of  machine
+
+        :param w: String to analysed during animation
+        :param filename: Name of gif (default is tm.gif")
+        :param symbols: Replacements of the symbols to show (default is empty)
+        :param states: Replacements of the states to show (default is empty)
+        :param dpi: Resolution of image (default is "90.0")
+        :param show: In interactive mode return gif
+        :param loop: Number of loops in annimation, cero is forever (default is 0)
+        :param duration: Duration in msegs among steps (default is 500)
+        :param max_steps: Maximum number of steps to consider (default is 1000)
+        :returns: None or HTML for Ipython"""
         dirpath = tempfile.mkdtemp()
         i=0
         images=[]
@@ -581,7 +639,7 @@ class TuringMachine():
             if ii>=max_steps:
                 break
             filename_=os.path.join(dirpath,f'{i}')
-            g=self.save_img(filename_,q_c=q,q_prev=set([self._state(q_c) for q_c,_,_,_ in q]),
+            g=self.save_img(filename_,q_c=q,q_prev=set([q_c for q_c,_,_,_ in q]),
                     symbols=symbols,states=states,status=status,
                     dpi=dpi)
             q_prev=q
@@ -591,11 +649,7 @@ class TuringMachine():
             images.append(im)
             i+=1
             filename_=os.path.join(dirpath,f'{i}')
-            if isinstance(q,set):
-                g=self.save_img(filename_,q_c=q,status=status,
-                        symbols=symbols,states=states,dpi=dpi)
-            else:
-                g=self.save_img(filename_,q_c=set([q]),status=status,
+            g=self.save_img(filename_,q_c=q,status=status,
                         symbols=symbols,states=states,dpi=dpi)
             im=Image.open(filename_+".png")
             images.append(im)
